Int

Struct Int 

pub struct Int(/* private fields */);
Mathematical integers for writting specifications. Mathematical integers are unbounded and arithmetic operation on them never over or underflow.

Implementations

Source

impl Int

pub fn pow2(self) -> Self

Raises 2 at the power self

pub fn _unsafe_from_str(s: &str) -> Self

Constructs a Int out of a string literal. This function assumes its argument consists only of decimal digits, with optionally a minus sign prefix.

pub fn rem_euclid(&self, v: Self) -> Self

Source

impl Int

pub fn to_u8(self) -> u8

Source

impl Int

pub fn to_u16(self) -> u16

Source

impl Int

pub fn to_u32(self) -> u32

Source

impl Int

pub fn to_u64(self) -> u64

Source

impl Int

pub fn to_u128(self) -> u128

Source

impl Int

pub fn to_usize(self) -> usize

Source

impl Int

pub fn to_i8(self) -> i8

Source

impl Int

pub fn to_i16(self) -> i16

Source

impl Int

pub fn to_i32(self) -> i32

Source

impl Int

pub fn to_i64(self) -> i64

Source

impl Int

pub fn to_i128(self) -> i128

Source

impl Int

pub fn to_isize(self) -> isize