pub struct Int(/* private fields */);Expand description
Mathematical integers for writting specifications. Mathematical integers are unbounded and arithmetic operation on them never over or underflow.
Implementations
Sourceimpl Int
impl Int
Sourcepub fn _unsafe_from_str(s: &str) -> Self
pub fn _unsafe_from_str(s: &str) -> Self
Constructs a Int out of a string literal. This function
assumes its argument consists only of decimal digits, with
optionally a minus sign prefix.
pub fn rem_euclid(&self, v: Self) -> Self